var services = [];

function addService(srv){
  srv.initialize(services.length);
  services.push(srv);
}


function requestFile( fileName, async, on_complete )
{
  var httpoj = new XMLHttpRequest();
  //Debug.writeLine(fileName);
  httpoj.open( "GET" , fileName , async );
  httpoj.onreadystatechange = function() { 
    if (httpoj.readyState==4) { 
      on_complete(httpoj);
    }
  };
  httpoj.send(null);
}


function Service(){
  this.initialize = function(_index){
    this.index = _index;
  };
  this.getChannelId = function(name){
    return chs[name][this.index];
  };
  // channel status setter/getter
  this.getStatus = function(name){
    return ( ch_status[name] & (1<<this.index) ) > 0;
  };
  this.setStatus = function(name){
    ch_status[name] |= (1<<this.index);
  };
  this.unsetStatus = function(name, index){
    ch_status[name] &= ~(1<<this.index);
  };
  // loading status counter
  this.ch_loading = new Object();
  this.incLoading = function(name){
    if(!this.ch_loading[name]) this.ch_loading[name] = 0;
    this.ch_loading[name]++;
  };
  this.decLoading = function(name){
    this.ch_loading[name]--;
  };
  this.getStatusClass = function(name){
    return (ch_status[name] > 0 ? 'online' : 'offline') + (this.ch_loading[name] > 0 ? ' loading' : '');
  };
  

  this.checkStatus = function(name, url, func){
    this.incLoading(name);
    updateFlyoutStatus(name);
    requestFile(url, true,
                function(oj){
                  var res  = oj.responseText;
                  var live = func(res);
                  if(live)
                    this.setStatus(name, this.index);
                  else
                    this.unsetStatus(name, this.index);
                  this.decLoading(name);
                  updateFlyoutStatus(name);
                }.bind(this));
  };
  this.checkStatusWithHTML = function(name, url, match_text){
    this.checkStatus(name, url, function(res){return res.match(match_text);});
  };
  // require implement in subclass (this is dummy)
  this.displayContent = function(ch_arr, w, h){};
  this.renewStatus = function(chs){};
};
var basicService = new Service();

// Ustream
function UstreamService(){
  this.ustChIds = new Object();
  this.setUstId = function(ch, id){
    this.ustChIds[ch] = id;
  };
  
  this.getUstId = function(ch){
    return this.ustChIds[ch];
  };
  
  this.getUstreamChannelId = function(ch) {
    if(!this.getUstId(ch)) {
      requestFile('http://www.ustream.tv/channel/' + ch, false,
                  function(oj){
                    var id;
                    var res  = oj.responseText;
                    id = res.match('channelid=([0-9]+)');
                    if(id)
                      this.setUstId(ch, id[1]);
                    else
                      if(id = res.match('rel="([0-9]+)"'))
                        this.setUstId(ch, id[1]);
                  }.bind(this));

    if(!this.getUstId(ch)) // if fail, use api
      requestFile('http://api.ustream.tv/xml/channel/' + 
                  encodeURIComponent(ch) +
                  '/getValueOf/id',
                  false,
                  function(oj) {
                    var id = oj.responseXML.selectSingleNode('/xml/results').text;
                    this.setUstId(ch, id);
                  }.bind(this));
    }
    return this.getUstId(ch);
  };
  
  this.displayContent = function(ch_arr, w, h){
    var ch = encodeURI(ch_arr[this.index]);
    var chid = this.getUstreamChannelId(ch);
    if(!chid)chid = '1';
    var url = 'http://www.ustream.tv/flash/live/1/' + chid;

    return '<object id="movie" width="'+w+'" height="'+h+'" type="application/x-shockwave-flash" classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000">' + 
      '<param name="movie" value="' + url +
      '"/><param name="flashvars" value="viewcount=true&amp;autoplay='+(autoplay?'true':'false')+'&amp;brand=embed"/>' +
      '<param name="wmode" value="transparent"/>' +
      '<param name="allowFullScreen" value="true"/>' +
      '<param name="allowscriptaccess" value="always"/></object>';
  };
  this.names = new Object();
  this.renewStatus = function(chs){
    var tmp = [];
    for(var name in chs){
      if(this.getChannelId(name)){
        tmp.push(this.getChannelId(name));
        this.names[this.getChannelId(name)] = name;
      }
    }
    var err = 0;
    for (var i = 0; i < tmp.length; i += 10){
      var curr = tmp.slice(i,i+10);
      //Debug.writeLine(curr.join('/'));
      if(curr.length == 1){
        this.checkStatus(names[curr[0]], 'http://api.ustream.tv/json/channel/'+encodeURI(curr[0])+'/getValueOf/status',
                    function(res){
                      eval('var a = '+res);
                      if(!a || !a.results){
                        err = 1;
                        return;
                      }
                      if(a.results.match('live'))
                        this.setStatus(names[curr[0]], this.index);
                      else
                        this.unsetStatus(names[curr[0]], this.index);
                      updateFlyoutStatus(names[curr[0]]);
                    }.bind(this));
      }else if(curr.length > 1){
        this.checkStatus('Revin', 'http://api.ustream.tv/json/channel/'+encodeURI(curr.join(';'))+'/getValueOf/status',
                    function(res){
                      //Debug.writeLine(res);
                      eval('results = '+res);
                      if(!results || !results.results){
                        err = 1;
                        return;
                      }
                      for(var i =0; i < results.results.length; i++){
                        var a = results.results[i];
                        var tn = this.names[a.uid];
                        if(a.result.match('live'))
                          this.setStatus(tn, this.index);
                        else
                          this.unsetStatus(tn, this.index);
                        updateFlyoutStatus(tn);
                      }
                    }.bind(this));
      }else{
        ;//err
      }
    }
    if(err) // fallback check
      for(var name in chs)
        if(this.getChannelId(name))
          this.checkStatusWithHTML(name,
                                   "http://www.ustream.tv/channel/"+encodeURI(chs[name][0]),
                                   "Live Broadcast started");
  };
};
UstreamService.prototype = basicService;


// Justin
function JustinService()
{
  this.displayContent = function(ch_arr, w, h){
    var ch = encodeURI(ch_arr[this.index]);
    
    return '<object type="application/x-shockwave-flash" height="'+h+'" width="'+w+'"' +
      'id="movie" data="http://www.justin.tv/widgets/live_embed_player.swf?channel='+ ch +'&amp;consumer_key=hQ3kBxqRXXBZJlPmu03JnA" bgcolor="#000000">' +
      '<param name="allowFullScreen" value="true" />' +
      '<param name="allowScriptAccess" value="always" />' + 
      '<param name="allowNetworking" value="all" />' +
      '<param name="wmode" value="transparent"/>' +
      '<param name="movie" value="http://www.justin.tv/widgets/live_embed_player.swf" />' +
      '<param name="flashvars" value="channel='+ ch +'&amp;auto_play='+(autoplay ? 'true' : 'false') +'&amp;start_volume=0.25&amp;consumer_key=hQ3kBxqRXXBZJlPmu03JnA" />' +
      '</object>';
  };
  
  this.renewStatus = function(chs){
    for(var name in chs){
      if(this.getChannelId(name))
        this.checkStatusWithHTML(name,
                            "http://www.justin.tv/"+encodeURI(chs[name][1]),
                            "PP\\['channel_live'\\] = true;");
    }
  };
}
JustinService.prototype = basicService;


// Blogtv
function BlogtvService()
{
  this.btvChInfos = new Object();
  this.setBtvInfo = function(ch, info){
    this.btvChInfos[ch] = info;
  };
  this.getBtvInfo = function(ch){
    return this.btvChInfos[ch];
  };
  this.getBlogtvChannelInfo = function(ch) {
    if(!this.getBtvInfo(ch)) {
      requestFile('http://www.blogtv.com/people/' + ch, false,
                  function(oj){
                    var mat;
                    var res  = oj.responseText;
                    var obj = new Object();
                    if(mat = res.match('var _liveEncEmbed="([0-9a-zA-Z]+)"')) obj.liveEnc = mat[1];
                    if(mat = res.match('var _OwnerProgramId=([0-9]+)')) obj.programId = mat[1];
                    if(mat = res.match('var _OwnerUserId=([0-9]+)')) obj.userId = mat[1];
                    if(mat = res.match('var _FMServerName *= *"([0-9a-z\.]+)"')) obj.serverName = mat[1];
                    if(mat = res.match('var _EdgeFMServerName *= *"([0-9a-z\.]+)"')) obj.edgeServerName = mat[1];
                    if(mat = res.match('var _channelId = "([0-9]+)"')) obj.channelId = mat[1];
                    this.setBtvInfo(ch, obj);
                  }.bind(this));
      
    }
    return this.getBtvInfo(ch);
  };
  
  this.displayContent = function(ch_arr, w, h){
    var ch = encodeURI(ch_arr[this.index]);
    var info = this.getBlogtvChannelInfo(ch);
    if( btvExp )
      return '<iframe src="http://www.blogtv.com/Flash/BTP.swf?iniLocation=http://www.blogtv.com/en/FXml/&liveEnc='
        + info.liveEnc+'&ApplicationName=BlogTV&ServerName='+info.serverName+'&EdgeServerName='+info.edgeServerName
          + '&BroadCasterUserid='+info.userId+'&ProgId='+info.programId+'&PlayerName='+ch+'&channelId='+info.channelId
            + '&chDescUrlAd=http://www.blogtv.com/people/'+ch+'&durationAd=2700&ps=0&ut=0&ts=0&fme=1" width="'+w+'" height="'+h+'" frameborder="0"/>';
    else
      return '<object id="movie" width="'+ w +'" height="'+ h +'" type="application/x-shockwave-flash"'
        + ' data="http://www.blogtv.com/livesdk/' + info.liveEnc + '">'
          + '<param name="movie" value="http://www.blogtv.com/livesdk/' + info.liveEnc + '"/>'
            + '<param name="allowFullScreen" value="true"/></object>';
  };
  
  this.renewStatus = function(chs){
    for(var name in chs){
      if(this.getChannelId(name)){
        var ch = encodeURI(this.getChannelId(name));
        this.checkStatusWithHTML(name,
                            "http://www.blogtv.com/Search.asp?SearchType=0&SearchStr="+ch,
                            "<table.*\'/People/"+ch+"\'.*http://storage\.blogtv\.com/Images/en/LIVE\.gif.*</table>");//
      }
    }
  };
};
BlogtvService.prototype = basicService;


addService(new UstreamService());
addService(new JustinService());
addService(new BlogtvService());
